//---------------------------------------------------------------------------
// StampMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Utilcls.h>
#include "Lib/Moochos/Moa/Dialogs.h"
#include "Lib/Moochos/Moa/Files.h"
#include "Lib/Moochos/Moa/Lists.h"
#include "Lib/Moochos/Moa/Shell.h"
#include "ProgressDialogUnit.h"
#include "StampMan.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace StampMan {
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString DateFormat = "yyyy-mm-dd";
const AnsiString TaskDirNameFormat = "%sRestamper Task #%03d (%s)";
const AnsiString LogFileNameFormat = "%sLog.txt";
const AnsiString BackupFileNameFormat = "%sBackup.dat";
const int MinTaskIndex = 1;
const int MaxTaskIndex = 999;
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TStampMan::Initialize()
{
	try {
		FOutputDir = GetSpecialDir(CSIDL_DESKTOPDIRECTORY);
		if(FOutputDir.Length() == 0) {
			throw EStampManError("The program cannot get the default output directory.");
		}

		FFilesDialog = new TOpenDialog(NULL);
		FilesDialog->Title = "Stamp";
		FilesDialog->Filter = "All|*";
		FilesDialog->Options =
			TOpenOptions() <<
			ofAllowMultiSelect <<
			ofPathMustExist <<
			ofFileMustExist;

		FListDialog = new TOpenDialog(NULL);
		ListDialog->Title = "Stamp";
		ListDialog->Filter = "File Name List (*.txt)|*.txt|All|*";
		ListDialog->Options =
			TOpenOptions() <<
			ofAllowMultiSelect <<
			ofPathMustExist <<
			ofFileMustExist;

		FRestoreDialog = new TOpenDialog(NULL);
		RestoreDialog->Title = "Restore";
		RestoreDialog->Filter = "Restamper Backup (*.dat)|*.dat|All|*";
		RestoreDialog->Options =
			TOpenOptions() <<
			ofPathMustExist <<
			ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::Uninitialize()
{
	delete FRestoreDialog;
	FRestoreDialog = NULL;

	delete FListDialog;
	FListDialog = NULL;

	delete FFilesDialog;
	FFilesDialog = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::CreateTaskDir(AnsiString& ALogFileName, AnsiString& ABackupFileName)
{
	AnsiString taskDir = AnsiString().sprintf(
		TaskDirNameFormat.c_str(),
		IncludeTrailingPathDelimiter(OutputDir).c_str(),
		TaskIndex,
		FormatDateTime(DateFormat, Now()).c_str());
	if(FileOrDirExists(taskDir) || taskDir.Length() == 0 || !ForceDirectories(taskDir)) {
		throw EStampManError(AnsiString().sprintf(
			"The program cannot create a task directory as \"%s\".\r\n\r\n"
			"If a file or directory with the name exists, move it and retry.",
			taskDir.c_str()));
	}
	TaskIndex = TaskIndex == MaxTaskIndex ? MinTaskIndex : (TaskIndex + 1);
	ALogFileName = AnsiString().sprintf(
		LogFileNameFormat.c_str(),
		IncludeTrailingPathDelimiter(taskDir).c_str());
	ABackupFileName = AnsiString().sprintf(
		BackupFileNameFormat.c_str(),
		IncludeTrailingPathDelimiter(taskDir).c_str());
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::Stamp(const TStampTaskArgs& AArgs)
{
	int warningCount;
	int errorCount;
	try {
		TPtr<TStampTask> task = new TStampTask(true, AArgs);
		FTask = task;
		FTask->OnStart = TaskStart;
		FTask->OnEnd = TaskEnd;
		FTask->OnStatus = TaskStatus;
		FTask->OnProgress = TaskProgress;
		FTask->OnCancel = TaskCancel;
		FTask->OnIdle = TaskIdle;
		FTask->OnException = TaskException;
		FTaskCanceling = false;

		ProgressDialog->Caption = "Stamp";
		ProgressDialog->Message = AnsiString();
		ProgressDialog->Progress = 0;
		ProgressDialog->OnExecute = ProgressDialogExecute;
		ProgressDialog->OnCancel = ProgressDialogCancel;
		ProgressDialog->Execute();

		warningCount = FTask->Log->WarningCount;
		errorCount = FTask->Log->ErrorCount;
	}
	__finally {
		FTask = NULL;
	}
	ReportTaskResults(AArgs, warningCount, errorCount);
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::ReportTaskResults(const TStampTaskArgs& AArgs, int AWarningCount, int AErrorCount)
{
	if(AWarningCount == 0 && AErrorCount == 0) {
		return;
	}
	if(Application->MessageBox(
		AnsiString().sprintf(
			"There are %d warnings and %d errors.\r\n\r\nOpen the log right now?",
			AWarningCount,
			AErrorCount).c_str(),
		Application->Title.c_str(),
		MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
		return;
	}
	SHELLEXECUTEINFO executeInfo = {0};
	executeInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	executeInfo.fMask = SEE_MASK_FLAG_NO_UI;
	executeInfo.hwnd = Application->Handle;
	executeInfo.lpFile = AArgs.LogFileName.c_str();
	executeInfo.nShow = SW_SHOWNORMAL;
	if(!ShellExecuteEx(&executeInfo)) {
		throw EStampManError("The program cannot open the log.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskStart(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskEnd(TObject* ASender)
{
	ProgressDialog->Finish();
	ProgressDialog->CloseDialog();
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskStatus(TObject* ASender, const AnsiString& AMessage)
{
	ProgressDialog->Message = AMessage;
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskProgress(TObject* ASender)
{
	ProgressDialog->Progress = FTask->Progress;
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskCancel(TObject* ASender)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskIdle(TObject* ASender)
{
	if(!FTask->Async) {
		Application->ProcessMessages();
	}
	if(FTaskCanceling && !FTask->Canceled) {
		FTaskCanceling = false;
		if(Application->MessageBox(
			"Cancel the task?",
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) == IDYES) {
			FTask->Cancel();
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::TaskException(TObject* ASender, Exception* AException)
{
	Application->ShowException(AException);
	ProgressDialog->Finish();
	ProgressDialog->CloseDialog();
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::ProgressDialogExecute(TObject* ASender)
{
	FTask->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::ProgressDialogCancel(TObject* ASender)
{
	FTaskCanceling = true;
}
//---------------------------------------------------------------------------
__fastcall TStampMan::TStampMan() :
	TObject(),
	FOutputDir(),
	FTaskIndex(MinTaskIndex),
	FTask(NULL),
	FTaskCanceling(false),
	FFilesDialog(NULL),
	FListDialog(NULL),
	FRestoreDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TStampMan::~TStampMan()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::StampFiles(const TFileTimeKinds AFileTimeKinds, const TFileTimes& AFileTimes, const TStampTaskOptions& AOptions)
{
	TPtr<TStrings> fileNames = new TStringList();
	FilesDialog->FileName = AnsiString();
	if(!FilesDialog->Execute()) {
		Abort();
	}
	fileNames->Assign(FilesDialog->Files);
	FilesDialog->Files->Clear();
	Stamp(AFileTimeKinds, AFileTimes, fileNames->Text, AOptions);
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::StampDirectory(const TFileTimeKinds AFileTimeKinds, const TFileTimes& AFileTimes, const TStampTaskOptions& AOptions)
{
	AnsiString directory;
	if(!ChooseDirectory("Choose the target.", directory)) {
		Abort();
	}
	Stamp(AFileTimeKinds, AFileTimes, directory, AOptions);
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::StampList(const TFileTimeKinds AFileTimeKinds, const TFileTimes& AFileTimes, const TStampTaskOptions& AOptions)
{
	ListDialog->FileName = AnsiString();
	if(!ListDialog->Execute()) {
		Abort();
	}
	try {
		TPtr<TStrings> fileNames = new TStringList();
		fileNames->LoadFromFile(ListDialog->FileName);
		Pack(fileNames);
		Stamp(AFileTimeKinds, AFileTimes, fileNames->Text, AOptions);
		ListDialog->Files->Clear();
	}
	catch(EStreamError& e) {
		throw EStampManError("The program cannot load file names.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::Stamp(const TFileTimeKinds AFileTimeKinds, const TFileTimes& AFileTimes, const AnsiString& AFileNames, const TStampTaskOptions& AOptions)
{
	TStampTaskArgs args;
	args.FileTimeKinds = AFileTimeKinds;
	args.FileTimes = AFileTimes;
	args.FileNames = AFileNames;
	CreateTaskDir(args.LogFileName, args.BackupFileName);
	args.Options = TStampTaskOptions(AOptions) >> stRestore;
	Stamp(args);
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::Restore()
{
	RestoreDialog->FileName = AnsiString();
	if(!RestoreDialog->Execute()) {
		Abort();
	}
	TStampTaskArgs args;
	args.SourceFileName = RestoreDialog->FileName;
	CreateTaskDir(args.LogFileName, args.BackupFileName);
	args.Options = TStampTaskOptions() << stRestore;
	Stamp(args);
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::OpenOutputDir()
{
	AnsiString outputDir = OutputDir;

	SHELLEXECUTEINFO executeInfo = {0};
	executeInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	executeInfo.fMask = SEE_MASK_FLAG_NO_UI;
	executeInfo.hwnd = Application->Handle;
	executeInfo.lpFile = outputDir.c_str();
	executeInfo.nShow = SW_SHOWNORMAL;
	if(!ShellExecuteEx(&executeInfo)) {
		throw EStampManError("The program cannot open the output directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampMan::ChangeOutputDir()
{
	AnsiString newOutputDir = OutputDir;
	if(!ChooseDirectory("Choose the new output directory.", newOutputDir)) {
		Abort();
	}
	OutputDir = newOutputDir;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::StampMan
//---------------------------------------------------------------------------

