//---------------------------------------------------------------------------
// StampTask.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "StampTask.h"
#include "StampCommand.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Restamper { namespace StampTask {
//---------------------------------------------------------------------------
void __fastcall TStampLog::Initialize()
{
	try {
		FStream = new TFileStream(FFileName, fmCreate | fmShareDenyWrite);
	}
	catch(EStreamError& e) {
		Uninitialize();
		throw EStampError("The program cannot create the log file.");
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampLog::Uninitialize()
{
	delete FStream;
	FStream = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TStampLog::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TStampLog::FormatMessage(const TDateTime& ADateTime, const AnsiString& ATag, const AnsiString& AMessage)
{
	return AnsiString().sprintf(
		"%s %s %s",
		FormatDateTime(DateTimeFormat, ADateTime).c_str(),
		ATag.c_str(),
		AMessage.c_str());
}
//---------------------------------------------------------------------------
void __fastcall TStampLog::WriteLine(const AnsiString& AText)
{
	AnsiString text = AText + LineBreak;
	FStream->Write(text.c_str(), text.Length());
}
//---------------------------------------------------------------------------
__fastcall TStampLog::TStampLog(const AnsiString& AFileName) :
	TObject(),
	FStream(NULL),
	FFileName(AFileName),
	FInfoTag("[Info]"),
	FWarningTag("[Warning]"),
	FErrorTag("[Error]"),
	FDateTimeFormat("yyyy-mm-dd hh:nn:ss"),
	FLineBreak("\r\n"),
	FInfoCount(0),
	FWarningCount(0),
	FErrorCount(0),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TStampLog::~TStampLog()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TStampLog::LogInfo(const AnsiString& AMessage)
{
	WriteLine(FormatMessage(Now(), InfoTag, AMessage));
	FInfoCount++;
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TStampLog::LogWarning(const AnsiString& AMessage)
{
	WriteLine(FormatMessage(Now(), WarningTag, AMessage));
	FWarningCount++;
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TStampLog::LogError(const AnsiString& AMessage)
{
	WriteLine(FormatMessage(Now(), ErrorTag, AMessage));
	FErrorCount++;
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::Initialize()
{
	try {
		FLog = new TStampLog(FArgs.LogFileName);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::Uninitialize()
{
	delete FLog;
	FLog = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::GetCommandArgsList(std::list<TStampCommandArgs>& ACommandArgsList)
{
	Log->LogInfo("Getting stamp command arguments list...");
	DoStatus("Getting stamp command arguments list...");
	std::list<TStampCommandArgs> newCommandArgsList;
	{
		TPtr<TStampSource> source = MakeStampSource(Args);
		while(!source->IsFinished()) {
			EnsureCanContinue();
			newCommandArgsList.push_back(source->Read());
			Idle();
		}
	}
	ACommandArgsList.swap(newCommandArgsList);
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::ProcessCommandArgsList(std::list<TStampCommandArgs>& ACommandArgsList)
{
	Log->LogInfo("Making backup file: " + Args.BackupFileName);
	DoStatus("Making backup file...");
	TPtr<TStampBackup> backup = new TStampBackup(Args);
	int current = 0;
	int all = ACommandArgsList.size();
	while(current < all) {
		EnsureCanContinue();
		ProcessCommandArgs(ACommandArgsList.front(), backup);
		ACommandArgsList.pop_front();
		SetProgress(++current * 100i64 / all);
		Idle();
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::ProcessCommandArgs(const TStampCommandArgs& ACommandArgs, TStampBackup* ABackup)
{
	DoStatus(ExtractFileName(ACommandArgs.FileName));
	TPtr<TStampCommand> command = new TStampCommand(ACommandArgs, ABackup);
	command->OnMakeBackupRecord = CommandMakeBackupRecord;
	command->OnGetFileAttrs = CommandGetFileAttrs;
	command->OnRemoveReadOnlyAttr = CommandRemoveReadOnlyAttr;
	command->OnSetFileTimes = CommandSetFileTimes;
	command->OnRestoreReadOnlyAttr = CommandRestoreReadOnlyAttr;
	command->OnSaveBackupRecord = CommandSaveBackupRecord;
	command->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::EnsureCanContinue()
{
	if(!Canceled) {
		return;
	}
	throw ETaskCancel("The task is canceled.");
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::CommandMakeBackupRecord(TObject* ASender, bool ADone, bool ASucceeded)
{
	if(!ADone) {
		Log->LogInfo("Making backup record of file: " +
			dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
	}
	else {
		if(ASucceeded) {
			// Do nothing
		}
		else {
			Log->LogError("Cannot make backup record of file: " +
				dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::CommandGetFileAttrs(TObject* ASender, bool ADone, bool ASucceeded)
{
	if(!ADone) {
		Log->LogInfo("Getting attributes of file: " +
			dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
	}
	else {
		if(ASucceeded) {
			// Do nothing
		}
		else {
			Log->LogError("Cannot get attributes of file: " +
				dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::CommandRemoveReadOnlyAttr(TObject* ASender, bool ADone, bool ASucceeded)
{
	if(!ADone) {
		Log->LogInfo("Removing read-only attribute from file: " +
			dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
	}
	else {
		if(ASucceeded) {
			// Do nothing
		}
		else {
			Log->LogError("Cannot remove read-only attribute from file: " +
				dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::CommandSetFileTimes(TObject* ASender, bool ADone, bool ASucceeded)
{
	if(!ADone) {
		Log->LogInfo("Setting times of file: " +
			dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
	}
	else {
		if(ASucceeded) {
			// Do nothing
		}
		else {
			Log->LogError("Cannot set times of file: " +
				dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::CommandRestoreReadOnlyAttr(TObject* ASender, bool ADone, bool ASucceeded)
{
	if(!ADone) {
		Log->LogInfo("Restoring read-only attribute of file: " +
			dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
	}
	else {
		if(ASucceeded) {
			// Do nothing
		}
		else {
			Log->LogError("Cannot restore read-only attribute of file: " +
				dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::CommandSaveBackupRecord(TObject* ASender, bool ADone, bool ASucceeded)
{
	if(!ADone) {
		Log->LogInfo("Saving backup record of file: " +
			dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
	}
	else {
		if(ASucceeded) {
			// Do nothing
		}
		else {
			Log->LogError("Cannot save backup record of file: " +
				dynamic_cast<TStampCommand*>(ASender)->Args.FileName);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::DoStatus(const AnsiString& AStatus)
{
	if(OnStatus != NULL) {
		Synchronize(DoStatusInMainThread, const_cast<AnsiString*>(&AStatus));
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::DoStatusInMainThread(void* AData)
{
	if(OnStatus != NULL) {
		OnStatus(this, *static_cast<AnsiString*>(AData));
	}
}
//---------------------------------------------------------------------------
void __fastcall TStampTask::Main()
{
	try {
		Log->LogInfo("Task started");
		DoStatus("Task started");

		std::list<TStampCommandArgs> commandArgsList;
		GetCommandArgsList(commandArgsList);
		ProcessCommandArgsList(commandArgsList);
		Log->LogInfo("Task completed");
		DoStatus("Task completed");
	}
	catch(ETaskCancel& e) {
		Log->LogWarning("Task canceled");
		DoStatus("Task canceled");
	}
	catch(Exception& e) {
		Log->LogError(e.Message);
		Log->LogWarning("Task aborted");
		DoStatus("Task aborted");
	}
	catch(...) {
		Log->LogError("Unknown error occured");
		Log->LogWarning("Task aborted");
		DoStatus("Task aborted");
	}
	Log->LogInfo("Warnings: " + AnsiString(Log->WarningCount));
	Log->LogInfo("Errors: " + AnsiString(Log->ErrorCount));
}
//---------------------------------------------------------------------------
__fastcall TStampTask::TStampTask(bool AAsync, const TStampTaskArgs& AArgs) :
	TTask(AAsync),
	FArgs(AArgs),
	FLog(NULL),
	FOnStatus(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TStampTask::~TStampTask()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Restamper::StampTask
//---------------------------------------------------------------------------

